﻿#include <algorithm>
#include <cmath>
#include <iostream>
#include <map>
#include <vector>
#include <numeric>
#include <queue>
#include <random>

using namespace std;

const int INF = 1e9 + 100;

struct edge {
	int w, color;
};

enum transition {
	ANY, MIN, MAX
};

vector<int> solve(int n, const vector<vector<edge>>& g) {
	vector<int> dist(n, INF);
	dist[0] = 0;
	vector<bool> used(n);
	queue<int> q;

	used[0] = true;
	q.push(0);

	vector<int> max_on_route(n, -INF);
	vector<int> min_on_route(n, INF);
	vector<int> max_diff(n);
	vector<int> dp_parent_min(n, -1);
	vector<int> dp_parent_max(n, -1);
	vector<pair<int, transition>> dp_parent(n, {-1, ANY});
	max_diff[0] = 0;

	while (!q.empty()) {
		int v = q.front();
		q.pop();
		for (const auto& [w, color] : g[v]) {
			if (used[w]) {
				continue;
			}
			used[w] = true;
			dist[w] = dist[v] + 1;
			q.push(w);
		}

		if (v == 0) {
			continue;
		}

		max_diff[v] = -INF;

		for (const auto& [parent, color] : g[v]) {
			if (dist[parent] != dist[v] - 1) {
				continue;
			}
			
			if (max_on_route[v] < max(max_on_route[parent], color)) {
				max_on_route[v] = max(max_on_route[parent], color);
				dp_parent_max[v] = parent;
			}

			if (min_on_route[v] > min(min_on_route[parent], color)) {
				min_on_route[v] = min(min_on_route[parent], color);
				dp_parent_min[v] = parent;
			}

			if (max_diff[v] < max_diff[parent]) {
				max_diff[v] = max_diff[parent];
				dp_parent[v] = {parent, ANY};
			}

			if (max_diff[v] < color - min_on_route[parent]) {
				max_diff[v] = color - min_on_route[parent];
				dp_parent[v] = {parent, MIN};
			}

			if (max_diff[v] < max_on_route[parent] - color) {
				max_diff[v] = max_on_route[parent] - color;
				dp_parent[v] = {parent, MAX};
			}
		}
	}

	int cur_v = n - 1;
	auto cur_transition = ANY;
	vector<int> path;
	while (cur_v != -1) {
		path.push_back(cur_v);
		if (cur_transition == ANY) {
			cur_transition = dp_parent[cur_v].second;
			cur_v = dp_parent[cur_v].first;
		} else if (cur_transition == MIN) {
			cur_v = dp_parent_min[cur_v];
		} else {
			cur_v = dp_parent_max[cur_v];
		}
	}
	reverse(path.begin(), path.end());
	return path;
}

int find_diff(const vector<int>& path, const vector<vector<edge>>& g) {
	int max_edge_color = -INF, min_edge_color = INF;

	for (size_t i = 0; i + 1 < path.size(); i++) {
		int u = path[i], v = path[i + 1];
		for (const auto& [v_edge, color] : g[v]) {
			if (v_edge == u) {
				max_edge_color = max(max_edge_color, color);
				min_edge_color = min(min_edge_color, color);
			}
		}
	}

	return max_edge_color - min_edge_color;
}

vector<int> solve_stupid(int n, const vector<vector<edge>>& g) {
	unsigned MAX_MASK = (1 << n);
	vector<int> best_path;
	int best_path_diff = -INF;

	for (unsigned orig_mask = 1; orig_mask < MAX_MASK; orig_mask++) {
		unsigned mask = orig_mask;
		vector<int> vertices;
		for (int i = 0; i < n; i++) {
			if (mask % 2 == 1) {
				vertices.push_back(i);
			}
			mask /= 2;
		}

		do {
			if (vertices.front() != 0 || vertices.back() != n - 1) {
				continue;
			}

			bool has_all_edges = true;

			for (size_t i = 0; i + 1 < vertices.size(); i++) {
				int u = vertices[i], v = vertices[i + 1];
				bool has_edge = false;
				for (const auto& [v_edge, _] : g[v]) {
					if (v_edge == u) {
						has_edge = true;
						break;
					}
				}
				if (!has_edge) {
					has_all_edges = false;
					break;
				}
			}

			if (!has_all_edges) {
				continue;
			}

			int diff = find_diff(vertices, g);

			
			if (
				best_path.empty() || 
				best_path.size() > vertices.size() ||
				(best_path.size() == vertices.size() && best_path_diff < diff)
			) {
				best_path = vertices;
				best_path_diff = diff;
			}

		} while (next_permutation(vertices.begin(), vertices.end()));
	}

	return best_path;
}

default_random_engine rnd;

vector<vector<edge>> generate_random_test(int n) {
	vector<pair<int, int>> all_edges;
	for (int i = 0; i < n; i++) {
		for (int j = i + 1; j < n; j++) {
			all_edges.push_back({i, j});
		}
	}
	shuffle(all_edges.begin(), all_edges.end(), rnd);
	int m = rnd() % all_edges.size();
	vector<vector<edge>> g(n);
	for (int i = 0; i < m; i++) {
		auto [u, v] = all_edges[i];
		int color = rnd() % ((int) 1e9);
		g[u].push_back({v, color});
		g[v].push_back({u, color});
	}
	return g;
}

/*
int main() {
	int n = 6;
	while (true) {
		auto g = generate_random_test(n);
		auto solution_stupid = solve_stupid(n, g);
		if (solution_stupid.size() == 0) {
			continue;
		}

		auto solution_smart = solve(n, g);
		int diff_stupid = find_diff(solution_stupid, g);
		int diff_smart = find_diff(solution_smart, g);
		if (solution_smart.size() != solution_stupid.size() || diff_stupid != diff_smart) {
			cout << "You're wrong!!" << endl;
			int x = 0;
		}
	}
}
*/


int main() {
	ios_base::sync_with_stdio(0); cin.tie(0); cout.tie(0);
	int n, m;
	cin >> n >> m;
	vector<vector<edge>> g(n);
	for (int i = 0; i < m; i++) {
		int u, v, c;
		cin >> u >> v >> c;
		u--; v--;
		g[u].push_back({v, c});
		g[v].push_back({u, c});
	}

	auto path = solve(n, g);
	cout << path.size() - 1 << "\n";
	for (int v : path) {
		cout << v + 1 << " ";
	}
}
